from jinja2 import Template
from mage_ai.shared.parsers import encode_complex
from typing import Dict
import os
import simplejson
import yaml


def build_config_json(
    absolute_file_path: str,
    variables: Dict,
) -> str:
    return simplejson.dumps(
        interpolate_variables_for_block_settings(
            absolute_file_path,
            variables,
        )['config'],
        default=encode_complex,
        ignore_nan=True,
    )


def interpolate_variables_for_block_settings(
    absolute_file_path: str,
    variables: Dict,
) -> Dict:
    with open(absolute_file_path, 'r') as f:
        return interpolate_variables(f.read(), variables)


def interpolate_variables(
    text: str,
    variables: Dict,
) -> Dict:
    settings_string = Template(text).render(
        env_var=os.getenv,
        variables=lambda x: variables.get(x),
    )

    return yaml.full_load(settings_string)




import json


def print_logs_from_output(output):
    from mage_integrations.utils.logger.constants import (
        LOG_LEVEL_ERROR,
        LOG_LEVEL_EXCEPTION,
        TYPE_LOG,
    )

    for line in output.split('\n'):
        try:
            data = json.loads(line)
            if TYPE_LOG == data.get('type'):
                print(json.dumps(data))
            if data.get('level') in [LOG_LEVEL_ERROR, LOG_LEVEL_EXCEPTION]:
                message = data.get('message')
                tags = data.get('tags')
                if message:
                    if tags:
                        message = f'{message} {json.dumps(tags)}'
                else:
                    message = 'Exception raised, please check logs for more details.'

                raise Exception(message)
        except json.decoder.JSONDecodeError:
            pass



SOURCES = [
    dict(name='Amplitude'),
    dict(name='BigQuery'),
    dict(name='Google Search Console'),
    dict(name='MySQL'),
    dict(name='PostgreSQL'),
    dict(name='Redshift'),
    dict(name='Salesforce'),
    dict(name='Stripe'),
]



from mage_ai.orchestration.constants import DATABASE_CONNECTION_URL_ENV_VAR
from mage_ai.shared.environments import is_test
from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker
import os

TEST_DB = 'test.db'

db_connection_url = os.getenv(DATABASE_CONNECTION_URL_ENV_VAR)
db_kwargs = dict(pool_pre_ping=True)

if not db_connection_url:
    if is_test():
        db_connection_url = f'sqlite:///{TEST_DB}'
    elif os.path.exists('mage_ai/orchestration/db/'):
        db_connection_url = 'sqlite:///mage_ai/orchestration/db/mage-ai.db'
    else:
        db_connection_url = 'sqlite:///mage-ai.db'
    db_kwargs['connect_args'] = {'check_same_thread': False}

engine = create_engine(
    db_connection_url,
    **db_kwargs,
)
session_factory = sessionmaker(bind=engine)


class DBConnection:
    def __init__(self):
        self.session = None

    def start_session(self):
        self.session = scoped_session(session_factory)

    def close_session(self):
        self.session.close()


db_connection = DBConnection()
